package cache

import (
	"context"
	"errors"
	"time"

	"github.com/go-redis/redis/v8"
	"go.uber.org/zap"
)

const queueName = "taskQueue"

var ErrNotFound = errors.New("value not found in cache")

type Cache struct {
	logger *zap.Logger
	redis  *redis.Client
}

func New(addr, port, pass string, db int, logger *zap.Logger) *Cache {
	rdb := redis.NewClient(&redis.Options{
		Addr:     addr + ":" + port,
		Password: pass,
		DB:       db,
	})

	return &Cache{
		logger: logger,
		redis:  rdb,
	}
}

func (c *Cache) Set(ctx context.Context, key, value string, ttl time.Duration) error {
	return c.redis.Set(ctx, key, value, ttl).Err()
}

func (c *Cache) Get(ctx context.Context, key string) (string, error) {
	res, err := c.redis.Get(ctx, key).Result()
	if err != nil {
		if err == redis.Nil {
			return "", ErrNotFound
		}
		return "", err
	}

	return res, nil
}

func (c *Cache) Enqueue(ctx context.Context, value string) error {
	return c.redis.RPush(ctx, queueName, value).Err()
}

func (c *Cache) Dequeue(ctx context.Context) (string, error) {
	res, err := c.redis.LPop(ctx, queueName).Result()
	if err != nil {
		return "", err
	}

	return res, nil
}
